#
# Create a polar bar chart
#
# Example use:
#   Let the represent a clock.  The data represent call volume
#   at a tech company help desk.  Radii of the wedges represent
#   the call volume.  Width of the wedges are a slice of time.
#   Color determined by wedge radius.
#
# Code from:
# https://matplotlib.org/stable/gallery/pie_and_polar_charts/polar_bar.html#sphx-glr-gallery-pie-and-polar-charts-polar-bar-py
# 
# Edited by David Mathias
# February 2021
#

import matplotlib.pyplot as plt
import numpy as np

# font sizes
LABELSIZE = 14
TICKSIZE = 12
TITLESIZE = 18

    
def main():
    # Fixing random state for reproducibility
    np.random.seed(19680801)

    # Number of wedges; one per hour
    N = 12
    
    # creates np array of angles at which wedges are centered - 
    # linspace creates evenly spaced values
    theta = np.linspace(0.0, 2 * np.pi, N, endpoint=False)
    
    # creates np array of radii in range 0.0 -- 99.99...
    radii = 100 * np.random.rand(N)
    
    # creates np array of wedge widths
    # we want each wedge to represent an hour
    width = 2*np.pi/12 * np.ones(12)
    
    # colors come from the specified colormap (cm.viridis)
    # are related to the radii of the wedges
    colors = plt.cm.viridis(radii / 100.0)

    # set for polar axes
    ax = plt.subplot(111, projection='polar')
    
    # create the plot
    #   align: align left edge of wedge with ticks
    #   color: the colormap used above
    #   alpha: degree of opacity
    pb = ax.bar(theta, radii, width=width, align='edge', color=colors, alpha=0.6)

    # make the labels around perimeter of plot
    # we want 12 evenly-spaced positions starting at 0 degrees and moving counterclockwise
    xtick_pos = [i * (2*np.pi/12) for i in range(N)]
    xtick_labels = ['3', '2', '1', '12', '11', '10', '9', '8', '7', '6', '5', '4']
    plt.xticks(xtick_pos, labels=xtick_labels)
    
    # set top of plot at 0.84 * height (measured from bottom left)
    # to make room for title
    plt.subplots_adjust(top=0.82)
    
    plt.xlabel('Call Volume per Hour', fontsize=LABELSIZE)
    
    # create the plot title
    plt.title('Help Desk Call Volume 7 AM to 7 PM\n', fontsize=TITLESIZE)

    # save and display plot
    plt.savefig('polar_barchart.png', dpi=300)
    plt.show()
    
    
if __name__ == '__main__':
    main()
    