#
# Creates a lineplot of target task demand by timestep for all 4 directions
# 
# Reads data from run.x.stepsummary
#

import sys
import csv
import os
from copy import deepcopy
from random import randint
import statistics
import itertools

import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
from matplotlib.lines import Line2D
from matplotlib.patches import Patch
import seaborn as sns


# write the calculated target demand data (movement of target in each dimension in each timestep)
# to a file in the run.x directory.  Name is run.x.targetdemand
def write_datafile(data, run):
    outfile = open('Output/' + run + '/' + run + '.targetdemand', 'w')
    for i in range(len(data[0])):
        outfile.write('T {} N {} E {} S {} W {} \n'.format(i, data[0][i], data[1][i], data[2][i], data[3][i]))
        
    outfile.close()
    

# fontsizes for plot elements
TITLESIZE = 18
LABELSIZE = 14
TICKSIZE = 12

# color palette for plot
col_list = ['bright green', 'tangerine', 'bright blue', 'bright red', 
            'green', 'cerulean', 'golden rod', 'bright pink']
# sns.palplot(sns.xkcd_palette(col_list))
plot_colors = sns.xkcd_palette(col_list)

# dictionary mapping int identifiers to task names
task_dict = {0:'North', 1:'East', 2:'South', 3:'West'}

# lists for avg switches per run, directory number, intensity variation, response probability
# Because seaborn wants long data, dir_nums, ivs, and rps are repeated as necessary to match
# with the avg switch values
north = []   
east = []
south = []
west = []

# run numnber to plot
run_num = 40
run = 'run.' + str(run_num)
data_fn = open('Output/' + run + '/' + run + '.stepsummary', 'r')

lines = data_fn.readlines()
data_fn.close()

# initialize prev to 0 and curr with the initial position of the target
prev_x = 0
prev_y = 0
elts = lines[0].split()
curr_x = float(elts[3])
curr_y = float(elts[4])

del lines[0]

for line in lines:
    elts = line.split()

    prev_x = curr_x
    prev_y = curr_y
    curr_x = float(elts[3])
    curr_y = float(elts[4])

    # determine distance moved by target in each dimention and update
    # the demand as the difference.
    if curr_x > prev_x:
        east.append(curr_x - prev_x)
        west.append(0.0)
    else:
        east.append(0.0)
        west.append(abs(curr_x - prev_x))

    if curr_y > prev_y:
        north.append(curr_y - prev_y)
        south.append(0.0)
    else:
        north.append(0.0)
        south.append(abs(curr_y - prev_y))

# convert the arrays into a 2D numpy array
data = np.asarray([north, east, south, west])

# write the data to a file
write_datafile(data, run)

# list of timesteps to serve as x values in plot
timesteps = [i for i in range(len(north))]
# number of timesteps, used to determine the increment between x-axis ticks
num_pts = len(timesteps)
# the increment between x ticks
tick_interval = num_pts / 10

# create the grid of subplots
fig, axes = plt.subplots(nrows=4, ncols=1, sharex=True, figsize=(10, 8))

# create the subplots
for i in range(len(task_dict)):
    sns.lineplot(ax=axes[i], y=data[i], x=timesteps, color=plot_colors[i]) #cmap='YlGnBu')
    axes[i].set_title(task_dict[i], fontsize=LABELSIZE)
    axes[i].set_ylim(0, 3.1)
    axes[i].set_xlim(-5, num_pts + 5)

# create the x labels
# x_labels = [str(i) if i % 50 == 0 else '' for i in range(len(data[0]))]
x_labels = [str(i) for i in range(num_pts + 5) if i % tick_interval == 0]
axes[0].set_xticks([i for i in range(num_pts + 5) if i % tick_interval == 0])
axes[0].set_xticklabels(x_labels, fontsize=TICKSIZE)

# this allows "global" axis labels (rather than labels for each subplot)
fig.add_subplot(111, frameon=False)
plt.tick_params(labelcolor='none', top=False, bottom=False, left=False, right=False)
plt.xlabel('Timesteps', fontsize=LABELSIZE)
plt.ylabel('Target Movement', fontsize=LABELSIZE)

axis_label_space = 0.20

fig.subplots_adjust(left=0.2,right=0.9)

plt.title('Task Demand per Timestep\n', fontsize=TITLESIZE)

plt.savefig('Output/' + run + '/target_demand_lineplot.png', dpi=300)

plt.show()
